#include "ccl.h"
#include "cudaBasicUtils.h"
#include "cuda/cclFunctions.h"

CCL::CCL()
{
	
}

CCL::~CCL()
{
}

bool CCL::FindRegions(int imgSizeX, int imgSizeY, CudaBuffer* segmentedData)
{
	if(!Init(imgSizeX, imgSizeY))
		return false;	
	
	//number of detected connected components (regions)
	m_numRegions = 0;
	m_segmentedData = segmentedData;

	//used for timing
	OnStartFindLabels();

	//call KERNEL 1 - compute local CCL solution
	if(!cclSolveCCLShared(sCudaBuffer2D(m_label2DData[0].GetData(), m_label2DData[0].GetPitch()),							
						  sCudaBuffer2D(m_segmentedData->GetData(), m_segmentedData->GetPitch()), 
						  m_threadsX, m_threadsY,
						  m_imgWidth, m_imgHeight, m_dataWidth, m_log2DataWidth))
		   return false;	
		
	//apply the merging scheme
	if(!cclMergeEquivalenceTrees
							   (sCudaBuffer2D(m_label2DData[0].GetData(), m_label2DData[0].GetPitch()),                                 
								sCudaBuffer2D(m_segmentedData->GetData(), m_segmentedData->GetPitch()), 
								(int*)m_int1Buf.GetData(),
								m_threadsX, m_threadsY,
								m_imgWidth, m_imgHeight, m_dataWidth, m_log2DataWidth, m_threadsX))
		return false;	
			
	//final update - flattening of the equivalence trees
	//Kernel 4
	if(!cclFlattenEquivalenceTrees(sCudaBuffer2D(m_label2DData[0].GetData(), m_label2DData[0].GetPitch()),								 
								 sCudaBuffer2D(m_label2DData[0].GetData(), m_label2DData[0].GetPitch()),
								 m_threadsX, m_threadsY,
								 m_imgWidth, m_imgHeight, m_dataWidth, m_log2DataWidth))
		 return false;			
	OnEndFindLabels();
	if(!m_label2DData[1].CopyFrom(&m_label2DData[0]))
			return false;
	//the labels are now merged and all continous areas are detected but the labels are not correctly indexed so we need to reindex them
	return ReindexLabels();
	
}
