#include "regionAnalyzer_kernels.cu"
#include "regionAnalyzerFunctions.h"
#include "cudaBasicUtils.h"

bool raComputeRegionsSizeShared(void* dOutRegSize, sCudaBuffer2D dRegionIn, int numRegions, int imgSizeX, int imgSizeY, int threadsX, int threadsY)
{
	dim3 block(threadsX, threadsY, 1);
	dim3 grid(imgSizeX/block.x, imgSizeY/block.y, 1);
	int threads = threadsX*threadsY;
	//compute the required number shared memory blocks based on the number of regions
	int sharedBlocks = (numRegions+threads-1)/threads;
	if(sharedBlocks == 1) raComputeRegionsSizeSharedKernel<1><<<grid, block, sizeof(int)*numRegions>>>((int*)dOutRegSize, (int*)dRegionIn.m_data, dRegionIn.m_pitch/sizeof(int), numRegions);
	else if(sharedBlocks == 2) raComputeRegionsSizeSharedKernel<2><<<grid, block, sizeof(int)*numRegions>>>((int*)dOutRegSize, (int*)dRegionIn.m_data, dRegionIn.m_pitch/sizeof(int), numRegions);
	else if(sharedBlocks == 3) raComputeRegionsSizeSharedKernel<3><<<grid, block, sizeof(int)*numRegions>>>((int*)dOutRegSize, (int*)dRegionIn.m_data, dRegionIn.m_pitch/sizeof(int), numRegions);
	else if(sharedBlocks == 4) raComputeRegionsSizeSharedKernel<4><<<grid, block, sizeof(int)*numRegions>>>((int*)dOutRegSize, (int*)dRegionIn.m_data, dRegionIn.m_pitch/sizeof(int), numRegions);
	else return false;
	return true;
}

bool raComputeRegionsSizeGlobal(void* dOutRegSize, sCudaBuffer2D dRegionIn, int imgSizeX, int imgSizeY, int threadsX, int threadsY)
{
	dim3 block(threadsX, threadsY, 1);
	dim3 grid(imgSizeX/block.x, imgSizeY/block.y, 1);
	raComputeRegionsSizeKernel<<<grid, block>>>((int*)dOutRegSize, (int*)dRegionIn.m_data, dRegionIn.m_pitch/sizeof(int));
	return true;
}

bool raComputeRegionsCentroidsShared(void* dOutRegCentroid, sCudaBuffer2D dRegionIn, int numRegions, int imgSizeX, int imgSizeY, int threadsX, int threadsY)
{
	dim3 block(threadsX, threadsY, 1);
	dim3 grid(imgSizeX/block.x, imgSizeY/block.y, 1);
	int threads = threadsX*threadsY;
	//compute the required number shared memory blocks based on the number of regions
	int sharedBlocks = (numRegions+threads-1)/threads;
	if(sharedBlocks == 1) raComputeRegionsCentroidSharedKernel<1><<<grid, block, sizeof(int2)*numRegions>>>((int2*)dOutRegCentroid, (int*)dRegionIn.m_data, dRegionIn.m_pitch/sizeof(int), numRegions);
	else if(sharedBlocks == 2) raComputeRegionsCentroidSharedKernel<2><<<grid, block, sizeof(int2)*numRegions>>>((int2*)dOutRegCentroid, (int*)dRegionIn.m_data, dRegionIn.m_pitch/sizeof(int), numRegions);
	else if(sharedBlocks == 3) raComputeRegionsCentroidSharedKernel<3><<<grid, block, sizeof(int2)*numRegions>>>((int2*)dOutRegCentroid, (int*)dRegionIn.m_data, dRegionIn.m_pitch/sizeof(int), numRegions);
	else if(sharedBlocks == 4) raComputeRegionsCentroidSharedKernel<4><<<grid, block, sizeof(int2)*numRegions>>>((int2*)dOutRegCentroid, (int*)dRegionIn.m_data, dRegionIn.m_pitch/sizeof(int), numRegions);
	else return false;
	return true;
}

bool raComputeRegionsCentroidsGlobal(void* dOutRegCentroid, sCudaBuffer2D dRegionIn, int imgSizeX, int imgSizeY, int threadsX, int threadsY)
{
	dim3 block(threadsX, threadsY, 1);
	dim3 grid(imgSizeX/block.x, imgSizeY/block.y, 1);
	raComputeRegionsCentroidGlobalKernel<<<grid, block>>>((int2*)dOutRegCentroid, (int*)dRegionIn.m_data, dRegionIn.m_pitch/sizeof(int));
	return true;
}

bool raComputeRegionType(void* dOutRegType, sCudaBuffer2D dRegionIn, sCudaBuffer2D dSegData, int numRegions, int imgSizeX, int imgSizeY, int threadsX, int threadsY)
{
	dim3 block(threadsX, threadsY, 1);
	dim3 grid(imgSizeX/block.x, imgSizeY/block.y, 1);
	raComputeRegionTypeKernel<<<grid, block>>>((uchar*)dOutRegType, (int*)dRegionIn.m_data, dRegionIn.m_pitch/sizeof(int), (uchar*)dSegData.m_data, dSegData.m_pitch/sizeof(uchar));
	return true;
}
