#include "segmentationFunctions.h"

texture<uchar4, 2, cudaReadModeElementType> imageTex;

__constant__ uchar c_segmentationIntervalData[MAX_SEGMENTATION_INTERVALS*6];

__global__ void applySegmentationKernel( uchar* dSegOut, int segPitch, 
								    int sizeX, int sizeY,
									int numSegments)
{
	uint x = threadIdx.x + blockIdx.x*blockDim.x;
	uint y = threadIdx.y + blockIdx.y*blockDim.y;	
	if(x >= sizeX || y >= sizeY) {		
		return;
	}
	uchar4 imgData = tex2D(imageTex, x, y);
	int segIntId = 0;
	int finalSegmentId = 0;
	for(int i=0;i<numSegments;++i) {
		if((imgData.x >= c_segmentationIntervalData[segIntId] && imgData.x <= c_segmentationIntervalData[segIntId+3]) &&
		   (imgData.y >= c_segmentationIntervalData[segIntId+1] && imgData.y <= c_segmentationIntervalData[segIntId+4]) &&
		   (imgData.z >= c_segmentationIntervalData[segIntId+2] && imgData.z <= c_segmentationIntervalData[segIntId+5]))
		{
			finalSegmentId = i+1;
		}
		segIntId += 6;
	}	
	dSegOut[x + y*segPitch] = finalSegmentId;
}
									

bool applySegmentation(sCudaBuffer2D bufOut, cudaArray* imgIn, int totalSizeX, int totalSizeY, int imgSizeX, int imgSizeY, int numSegments, unsigned char* segData)
{	
	cudaError_t err = cudaMemcpyToSymbol(c_segmentationIntervalData, segData, sizeof(uchar)*6*numSegments,0);
	if(err != cudaSuccess)
		return false;	
	cudaChannelFormatDesc channelDesc = cudaCreateChannelDesc<uchar4>();
	err = cudaBindTextureToArray(&imageTex, imgIn, &channelDesc);
	if(err != cudaSuccess) 
		return false;
	dim3 block(16, 16, 1);
    dim3 grid(totalSizeX / block.x, totalSizeY / block.y, 1);
	applySegmentationKernel<<<grid,block>>>((uchar*)bufOut.m_data, bufOut.m_pitch/sizeof(uchar), imgSizeX, imgSizeY, numSegments);

	err = cudaUnbindTexture(&imageTex);
	if(err != cudaSuccess) 
		return false;
	return true;
}

