#ifndef _CUDA_RESOURCE_H_
#define _CUDA_RESOURCE_H_

struct cudaGraphicsResource;

#include <GL/glew.h>
#include "cudaBuffer.h"
#include "cudaArray.h"

enum eCudaResourceMapFlags
{
	eCUDA_RESOURCE_MAP_FLAGS_NONE,
	eCUDA_RESOURCE_MAP_FLAGS_READ_ONLY,
	eCUDA_RESOURCE_MAP_FLAGS_WRITE_DISCARD
};

class CudaResource
{
public:
	CudaResource();
	~CudaResource();
	bool Unregister();
	bool RegisterTexture(GLenum texTarget, GLuint texId, int w, int h, int d, eCudaResourceMapFlags mapFlags);	
	bool ChangeMapFlags(eCudaResourceMapFlags mapFlags);
	bool MapResource();
	bool UnmapResource();
	CudaArray* GetMappedArray();
protected:
	void ClearMappedData();
private:
	cudaGraphicsResource* m_resource;
	CudaBuffer* m_mappedBuffer;		//mapped buffer in case the resource is a buffer
	CudaArray* m_mappedArray;		//mapped array in case the resouce is an array

	//original texture data
	GLenum m_texTarget;
	int m_texDim[3];
};

#endif