#include <stdlib.h>
#include <GL/glew.h>
#include <GL/glut.h>
#include <stdio.h>
#include "scene.h"
#include "timer.h"
#include <cuda_runtime_api.h>
#include <cuda_gl_interop.h>

Scene* g_scene;
int g_mainWindowId;
Timer g_fpsTimer;
int g_frames;
char g_buffer[200];

void display();
void idle();
void reshape(int w, int h);
void mouseFunc(int button, int state, int x, int y);
void specialFunc(int key, int x, int y);
void keyFunc(unsigned char key, int x, int y);

bool initCUDA();

int main(int argc, char** argv)
{
	int initScene = 0;
	int solver = 0;
	int res = 0;
	if(argc > 1) res = atoi(argv[1]);	
	Scene scene(initScene, solver, res);
	g_scene = &scene;
	//init GLUT
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGBA | GLUT_ALPHA | GLUT_DEPTH | GLUT_DOUBLE);	
	//init CUDA
	if(!initCUDA()) {
		printf("Error: Faild to initialize the CUDA library\n");
		system("pause");
		return -1;
	}
	glutInitWindowPosition(200, 50);
	glutInitWindowSize(g_scene->GetWinWidth(), g_scene->GetWinHeight());
	g_mainWindowId = glutCreateWindow("Find connected regions DEMO");
			
	if(!scene.Init()) {
		printf("Error: Failed to initialize the main scene\n");
		return -1;
	}

	glutDisplayFunc(display);
	glutIdleFunc(idle);
	glutReshapeFunc(reshape);	
	glutMouseFunc(mouseFunc);
	glutKeyboardFunc(keyFunc);
	glutSpecialFunc(specialFunc);
	g_fpsTimer.Start();
	g_frames = 0;
	glutMainLoop();
	return 0;
}

bool initCUDA()
{
	cudaDeviceProp prop;
	cudaGetDeviceProperties(&prop, 0);
	if(prop.major == 1 && prop.minor < 3) {
		//we use atomics on both shared and global memory .. only devices of compute capability 1.3 or higher supports them
		printf("Error: Your main CUDA device does not support the required functionality.\nA device with compute capability 1.3 or better is required\n");
		return false;
	}
	cudaError_t err = cudaGLSetGLDevice(0);
	if(err != cudaSuccess)
		return false;
	return true;
}

void display() {		
	g_scene->Display();

	//compute and display FPS
	g_frames++;
	clock_t time = g_fpsTimer.GetTime();
	if(time > 1000) {
		float tS = (float)time/1000.0f;
		sprintf(g_buffer, "Find connected regions DEMO, FPS = %f", (float)g_frames/tS);
		glutSetWindowTitle(g_buffer);
		g_frames = 0;
		g_fpsTimer.Start();
	}
}

void idle() {
	glutPostRedisplay();		
}

void reshape(int w, int h)
{
	g_scene->Reshape(w, h);
}

void mouseFunc(int button, int state, int x, int y)
{
	g_scene->OnMouseButton(button, state, x, y);
}

void keyFunc(unsigned char key, int x, int y)
{
	g_scene->OnKey(key);
}

void specialFunc(int key, int x, int y)
{
	g_scene->OnSpecialKey(key);;
}