#ifndef _REGION_ANALYZER_H_
#define _REGION_ANALYZER_H_

#include <GL/glew.h>
#include "cudaBuffer.h"
#include "cudaScan.h"
#include "cudaResource.h"

//class performing an analyzis of detected regions (right now it computes region sizes and centroids)

class RegionAnalyzer
{
public:
	RegionAnalyzer();
	~RegionAnalyzer();

	bool AnalyzeRegions(CudaBuffer* regionData, CudaBuffer* segmentationData, int numRegions, int imgW, int imgH, int dataW, int dataH);
	bool BindRegionTypeTexture(int unit);

	int GetRegionType(int regId);
	int GetRegionSize(int regId);
	void GetRegionCentroid(int regId, float& x, float& y);
private:
	bool Init(int numRegions, int dataW, int dataH);	
	bool InitTextures(int numRegions);
	bool TransferDataToTextures(int numRegions);
private:
	//compacted buffers - regions
	CudaBuffer m_regionSize;	//size of the regions
	CudaBuffer m_regionCentroid;
	CudaBuffer m_regionType;	//type of the region based on the input segmentation data
	int m_maxRegions;


	//host buffers
	CudaBuffer m_int1HostBuf;
	CudaBuffer m_regionSizeHost;	//size of the regions
	CudaBuffer m_regionCentroidHost;
	CudaBuffer m_regionTypeHost;	

	CudaScan m_scan;

	int m_threadsX, m_threadsY;
	int m_dataW, m_dataH;
	int m_log2DataW;
	int m_imgW, m_imgH;

	//OpenGL textures that are used to store some of the computed data
	GLuint m_regionTypeTexId;
	CudaResource m_regionTypeRes;
};

#endif