#ifndef _SCENE_H_
#define _SCENE_H_

#include <GL/glew.h>
#include "fboClass/renderBuffer.h"
#include "shaderCg.h"
#include "segmentation.h"
#include "ccl.h"
#include "regionAnalyzer.h"
#include "timer.h"

// the main class that handles all visualization and user input

class Scene
{
public:
	Scene(int initScene, int solver, int res);
	~Scene();
	bool Init();		
	void Reshape(int w, int h);
	void Display();
	void DisplaySubWindow(int winX, int winY, int winW, int winH);
	void OnMouseButton(int button, int state, int x, int y);
	void OnKey(int key);
	void OnSpecialKey(int key);

	int GetWinWidth() const { return m_winW; }
	int GetWinHeight() const { return m_winH; }	
private:
	bool InitGL();
	bool InitScene();
	bool InitShaders();
	bool ResizeRttTex();
	bool AnalyzeScene();
	void RenderScene();
	void RenderSceneDemo1();
	void RenderSceneDemo2();
	void RenderSceneDemo3();
	void RenderSceneDemo4();	
	void DisplayRttTex(int winX, int winY, int winW, int winH);
	void DisplayLabels();
	bool SelectRegionOnPos(int x, int y);
	float GetSelectedRegionTransparency();
	void DisplayText(int x, int y, void* font, char* text, bool rightAlign = false);
	void DisplayOffsetedText(int x, int y, void* font, char* text);

	void DisplayCentroids();
	void DisplayCentroid(int regId);	
	void DisplayQuad(int minX, int minY, int maxX, int maxY);
private:
	//openGL data
	GLuint m_rttTexId;
	GLuint m_regionsTexId;
	int m_rttTexW, m_rttTexH;
	FramebufferObject* m_fbo;
	Renderbuffer* m_rb;
	ShaderCg* m_displayObjectShader;
	ShaderCg* m_displaySceneShader;
	ShaderCg* m_displaySceneSubWShader;
	ShaderCg* m_displayColorShader;

	//size of the main window
	int m_winW, m_winH;
	
	//classes and data used for the CCL analysis
	Segmentation m_segmentation;
	CCLBase* m_ccl;
	RegionAnalyzer m_ra;
	CudaResource m_regionsTexRes;
	
	int m_selectedRegionId;	
	float m_selectedRegionBlinkingFrequency;
	float m_displayedRegionsTransparency;

	bool m_displayCentroids;
	bool m_displayRegions;
	bool m_displayHelp;
	bool m_displayLabels;
	
	int m_selectedSceneId;
	//resolution of the scene that is being analyzed
	int m_analysisResolutionX;
	int m_analysisResolutionY;

	//some timers used for animations
	Timer m_timer;
	Timer m_selectedRegionTimer;
};

#endif