#ifndef _SEGMENTATION_H_
#define _SEGMENTATION_H_

//segmentation of the input image data based on thresholding

#include <vector>
#include <GL/glew.h>
#include "cudaResource.h"
#include "cudaBuffer.h"

struct SegmentationInterval
{
	void SetMin(unsigned char minR, unsigned char minG, unsigned char minB) { m_minVal[0] = minR; m_minVal[1] = minG; m_minVal[2] = minB; }
	void SetMax(unsigned char maxR, unsigned char maxG, unsigned char maxB) { m_maxVal[0] = maxR; m_maxVal[1] = maxG; m_maxVal[2] = maxB; }
	void SetColor(unsigned char r, unsigned char g, unsigned char b) { m_color[0] = r; m_color[1] = g; m_color[2] = b; }
	unsigned char m_minVal[3];		//min value for r/g/b
	unsigned char m_maxVal[3];		//max value for r/g/b

	unsigned char m_color[3];		//a color in whis is the region of a given segment displayed
};

class Segmentation
{
public:
	Segmentation();
	~Segmentation();
	bool Init(GLenum target, GLuint texId, int w, int h);
	bool ApplySegmentation(int screenW, int screenH);
	CudaBuffer* GetSegmentedData() { return &m_segmentedData; }
	bool BindSegColorTex(int unit);
	SegmentationInterval* GetSegmentationInterval(int i) { return &m_segInterval[i]; }
	void InvalidateData() { m_dataInvalid = true; }
private:
	bool InitDefaultIntervals();
	bool CreateSegTextures();
	bool UpdateSegTextures();
private:
	std::vector<SegmentationInterval> m_segInterval;
	const int m_maxSegIntervals;
	CudaResource m_sceneTexRes;
	CudaBuffer m_segmentedData;		//an output buffer containing the segmented data	

	//opengl textures that are used to store some data about individual segments
	int m_allocatedSegs;
	GLuint m_segColorTexId;
	bool m_dataInvalid;
};

#endif