#include "shader.h"

Shader* Shader::m_activeShader = NULL;

Shader::~Shader() {	
	for(ShaderUniform1iIterator sI=m_sU1iMap.begin();sI!=m_sU1iMap.end();sI++) delete sI->second;
	for(ShaderUniform1fIterator sI=m_sU1fMap.begin();sI!=m_sU1fMap.end();sI++) delete sI->second;
	for(ShaderUniform2fIterator sI=m_sU2fMap.begin();sI!=m_sU2fMap.end();sI++) delete sI->second;
	for(ShaderUniform3fIterator sI=m_sU3fMap.begin();sI!=m_sU3fMap.end();sI++) delete sI->second;
	for(ShaderUniform4fIterator sI=m_sU4fMap.begin();sI!=m_sU4fMap.end();sI++) delete sI->second;
}

void Shader::DisableShaders() {
	if(m_activeShader != NULL) {
		m_activeShader->Unbind();
	} else {
		glUseProgramObjectARB(0);
		m_activeShader = NULL;
	}	
}

void Shader::ChangeUniform1i(const std::string& name, eShaderType type, GLint value) {
	//check if the value is in the map
	ShaderUniform1iIterator sI=m_sU1iMap.find(name);
	if(sI==m_sU1iMap.end())
	{
		ShaderUniform1i* sU=new ShaderUniform1i;	
		sU->m_value=value;
		m_sU1iMap[name]=sU;
	}
	else 
	{
		ShaderUniform1i* sU=sI->second;
		if(sU->m_value!=value) 
		{		
			sU->m_value=value;          
		}  else {
			return;		//the value is already set
		}
	}    
	SetUniform1i(name, type, value);
}


void Shader::ChangeUniform2i(const std::string& name, eShaderType type, GLint value0, GLint value1) {
	//check if the value is in the map
	ShaderUniform2iIterator sI=m_sU2iMap.find(name);
	if(sI==m_sU2iMap.end())
	{
		ShaderUniform2i* sU = new ShaderUniform2i;	
		sU->m_value[0] = value0;
		sU->m_value[1] = value1;
		m_sU2iMap[name]=sU;
	}
	else 
	{
		ShaderUniform2i* sU=sI->second;
		if(sU->m_value[0] != value0 || sU->m_value[1] != value1) 
		{		
			sU->m_value[0] = value0;          
			sU->m_value[1] = value1;          
		}  else {
			return;		//the value is already set
		}
	}    
	SetUniform2i(name, type, value0, value1);
}

void Shader::ChangeUniform1f(const std::string& name, eShaderType type, GLfloat value) {
	//check if the value is in the map
	ShaderUniform1fIterator sI=m_sU1fMap.find(name);
	if(sI==m_sU1fMap.end())
	{
		ShaderUniform1f* sU=new ShaderUniform1f;	
		sU->m_value=value;
		m_sU1fMap[name]=sU;
	}
	else 
	{
		ShaderUniform1f* sU=sI->second;
		if(sU->m_value!=value) 
		{		
			sU->m_value=value;          
		}  else {
			return;		//the value is already set
		}
	}    
	SetUniform1f(name, type, value);
}

void Shader::ChangeUniform2f(const std::string& name, eShaderType type, GLfloat value0, GLfloat value1) {
	//check if the value is in the map
	ShaderUniform2fIterator sI=m_sU2fMap.find(name);
	if(sI==m_sU2fMap.end())
	{
		ShaderUniform2f* sU = new ShaderUniform2f;	
		sU->m_value[0] = value0;
		sU->m_value[1] = value1;
		m_sU2fMap[name]=sU;
	}
	else 
	{
		ShaderUniform2f* sU=sI->second;
		if(sU->m_value[0] != value0 || sU->m_value[1] != value1) 
		{		
			sU->m_value[0] = value0;          
			sU->m_value[1] = value1;          
		}  else {
			return;		//the value is already set
		}
	}    
	SetUniform2f(name, type, value0, value1);
}

void Shader::ChangeUniform3f(const std::string& name, eShaderType type, GLfloat value0, GLfloat value1, GLfloat value2) {
	//check if the value is in the map
	ShaderUniform3fIterator sI=m_sU3fMap.find(name);
	if(sI==m_sU3fMap.end())
	{
		ShaderUniform3f* sU = new ShaderUniform3f;	
		sU->m_value[0] = value0;
		sU->m_value[1] = value1;
		sU->m_value[2] = value2;
		m_sU3fMap[name]=sU;
	}
	else 
	{
		ShaderUniform3f* sU=sI->second;
		if(sU->m_value[0] != value0 || sU->m_value[1] != value1 || sU->m_value[2] != value2) 
		{		
			sU->m_value[0] = value0;          
			sU->m_value[1] = value1;          
			sU->m_value[2] = value2;          
		}  else {
			return;		//the value is already set
		}
	}    
	SetUniform3f(name, type, value0, value1, value2);
}


void Shader::ChangeUniform4f(const std::string& name, eShaderType type, GLfloat value0, GLfloat value1, GLfloat value2, GLfloat value3) {
	//check if the value is in the map
	ShaderUniform4fIterator sI=m_sU4fMap.find(name);
	if(sI==m_sU4fMap.end())
	{
		ShaderUniform4f* sU = new ShaderUniform4f;	
		sU->m_value[0] = value0;
		sU->m_value[1] = value1;
		sU->m_value[2] = value2;
		m_sU4fMap[name]=sU;
	}
	else 
	{
		ShaderUniform4f* sU=sI->second;
		if(sU->m_value[0] != value0 || sU->m_value[1] != value1 || sU->m_value[2] != value2 || sU->m_value[3] != value3) 
		{		
			sU->m_value[0] = value0;          
			sU->m_value[1] = value1;          
			sU->m_value[2] = value2;          
			sU->m_value[3] = value3;  
		}  else {
			return;		//the value is already set
		}
	}    
	SetUniform4f(name, type, value0, value1, value2, value3);
}

void Shader::ChangeUniform1fArray(const std::string& name, eShaderType type, int count, const float* data)
{
	//no caching right now : TODO
	SetUniform1fArray(name, type, count, data);
}

void Shader::ChangeUniform1iArray(const std::string& name, eShaderType type, int count, const int* data)
{
	//no caching: TODO:
	SetUniform1iArray(name, type, count ,data);
};

void Shader::ChangeUniform2fArray(const std::string& name, eShaderType type, int count, const float* data)
{
	SetUniform2fArray(name, type, count, data);
}

void Shader::ChangeUniform3fArray(const std::string& name, eShaderType type, int count, const float* data)
{
	SetUniform3fArray(name, type, count, data);
}


void Shader::ChangeGLMatrix(const std::string& name, eShaderType type,  GLenum matrix, GLenum transform)
{
	SetGLMatrix(name, type,  matrix, transform);
}