#ifndef _SHADER_H_
#define _SHADER_H_

#include <string>
#include <map>
#include <GL/glew.h>
#include <gl/glut.h> //fonts are needed

//some structs for storing shader uniforms
//currently not all possible..just some that are used frequently

struct ShaderUniform
{
};

struct ShaderUniform1i:public ShaderUniform {
	GLint m_value;
};  

struct ShaderUniform2i:public ShaderUniform {
	GLint m_value[2];
};   

struct ShaderUniform1f:public ShaderUniform {
	GLfloat m_value;
};

struct ShaderUniform2f:public ShaderUniform {
	GLfloat m_value[2];
};  

struct ShaderUniform3f:public ShaderUniform {
	GLfloat m_value[3];
};   

struct ShaderUniform4f:public ShaderUniform {
	GLfloat m_value[4];
};   

typedef std::map<std::string,ShaderUniform1i*> ShaderUniform1iMap;
typedef ShaderUniform1iMap::iterator ShaderUniform1iIterator;

typedef std::map<std::string,ShaderUniform2i*> ShaderUniform2iMap;
typedef ShaderUniform2iMap::iterator ShaderUniform2iIterator;

typedef std::map<std::string,ShaderUniform1f*> ShaderUniform1fMap;
typedef ShaderUniform1fMap::iterator ShaderUniform1fIterator;

typedef std::map<std::string,ShaderUniform2f*> ShaderUniform2fMap;
typedef ShaderUniform2fMap::iterator ShaderUniform2fIterator;

typedef std::map<std::string,ShaderUniform3f*> ShaderUniform3fMap;
typedef ShaderUniform3fMap::iterator ShaderUniform3fIterator;

typedef std::map<std::string,ShaderUniform4f*> ShaderUniform4fMap;
typedef ShaderUniform4fMap::iterator ShaderUniform4fIterator;

enum eShaderType {
	GPULIB_VERTEX_SHADER,
	GPULIB_FRAGMENT_SHADER,
	GPULIB_GEOMETRY_SHADER
};


class Shader 
{
public:	
	Shader() {}
	virtual ~Shader();

	virtual bool LoadFromText(const char* vertexShader,const char* fragmentShader,char* vertexMain=0,char* fragmentMain=0) = 0;
	virtual bool LoadFromFile(const char* vertexShaderFile,const char* fragmentShaderFile,char* vertexMain=0,char* fragmentMain=0) = 0;
	//2. version ..loads also geometry shader
	virtual bool LoadFromFileWithGS(const char* vertexShaderFile,const char* geometryShaderFile, const char* fragmentShaderFile,char* vertexMain=0,char* geometryMain=0,char* fragmentMain=0) = 0;

	virtual void Bind() = 0;	
	static void DisableShaders();

	void ChangeUniform1i(const std::string& name, eShaderType type, GLint value);
	void ChangeUniform2i(const std::string& name, eShaderType type, GLint value0, GLint value1);
	void ChangeUniform1f(const std::string& name, eShaderType type, GLfloat value);  
	void ChangeUniform2f(const std::string& name, eShaderType type, GLfloat value0, GLfloat value1);  
	void ChangeUniform3f(const std::string& name, eShaderType type, GLfloat value0, GLfloat value1, GLfloat value2);  
	void ChangeUniform4f(const std::string& name, eShaderType type, GLfloat value0, GLfloat value1, GLfloat value2, GLfloat value3);  
	void ChangeUniform1fArray(const std::string& name, eShaderType type, int count, const float* data);
	void ChangeUniform1iArray(const std::string& name, eShaderType type, int count, const int* data);
	void ChangeUniform2fArray(const std::string& name, eShaderType type, int count, const float* data);
	void ChangeUniform3fArray(const std::string& name, eShaderType type, int count, const float* data);
	void ChangeGLMatrix(const std::string& name, eShaderType type,  GLenum matrix, GLenum transform);


	static Shader* GetActiveShader() { return m_activeShader; }
protected:
	virtual void Unbind() = 0;
	static void SetActiveShader(Shader* shader) { m_activeShader = shader; }
private:
	virtual bool SetUniform1i(const std::string& name, eShaderType type, GLint value) = 0;
	virtual bool SetUniform2i(const std::string& name, eShaderType type, GLint value0, GLint value1) = 0;
	virtual bool SetUniform1f(const std::string& name, eShaderType type, GLfloat value) = 0;
	virtual bool SetUniform2f(const std::string& name, eShaderType type, GLfloat value0, GLfloat value1) = 0;
	virtual bool SetUniform3f(const std::string& name, eShaderType type, GLfloat value0, GLfloat value1, GLfloat value2) = 0;
	virtual bool SetUniform4f(const std::string& name, eShaderType type, GLfloat value0, GLfloat value1, GLfloat value2, GLfloat value3) = 0;
	virtual bool SetUniform1fArray(const std::string& name, eShaderType type, int count, const float* data) = 0;
	virtual bool SetUniform1iArray(const std::string& name, eShaderType type, int count, const int* data) = 0;
	virtual bool SetUniform2fArray(const std::string& name, eShaderType type, int count, const float* data) = 0;
	virtual bool SetUniform3fArray(const std::string& name, eShaderType type, int count, const float* data) = 0;
	virtual bool SetGLMatrix(const std::string& name, eShaderType type,  GLenum matrix, GLenum transform) = 0;

//members
	//note that right now the uniforms are same for all shader types
	ShaderUniform1iMap m_sU1iMap;
	ShaderUniform2iMap m_sU2iMap;
	ShaderUniform1fMap m_sU1fMap;
	ShaderUniform2fMap m_sU2fMap;
	ShaderUniform3fMap m_sU3fMap;
	ShaderUniform4fMap m_sU4fMap;
protected:
	static Shader* m_activeShader;
};

#endif