#ifndef _CG_SHADER_H_
#define _CG_SHADER_H_

// class for handling cg shaders

#include "shader.h"
#include <Cg/cg.h>
#include <Cg/cgGL.h>
#include <map>
#include <string>

typedef std::map<std::string, CGparameter> CgParamMap;

class ShaderCg : public Shader {
public:
	ShaderCg();
	virtual ~ShaderCg();

	virtual void Bind();
	virtual bool LoadFromText(const char* vertexShader,const char* fragmentShader,char* vertexMain=0,char* fragmentMain=0);
	virtual bool LoadFromFile(const char* vertexShaderFile,const char* fragmentShaderFile,char* vertexMain=0,char* fragmentMain=0);
	//2. version ..loads also geometry shader
	virtual bool LoadFromFileWithGS(const char* vertexShaderFile,const char* geometryShaderFile, const char* fragmentShaderFile,char* vertexMain=0,char* geometryMain=0,char* fragmentMain=0);

private:
	virtual void Unbind();
	bool IsVertexShaderActive() const { return m_vertexHandle!=0; }
	bool IsGeometryShaderActive() const { return m_geometryHandle!=0; }
	bool IsFragmentShaderActive() const { return m_fragmentHandle!=0; }
	bool InitContext();

	CGparameter InitVertexParameter(const std::string& name);
	CGparameter GetVertexParameter(const std::string& name);
	CGparameter InitFragmentParameter(const std::string& name);
	CGparameter GetFragmentParameter(const std::string& name);
	CGparameter GetParameter(const std::string& name, eShaderType type);
	virtual bool SetUniform1i(const std::string& name, eShaderType type, GLint value);
	virtual bool SetUniform2i(const std::string& name, eShaderType type, GLint value0, GLint value1);
	virtual bool SetUniform1f(const std::string& name, eShaderType type, GLfloat value);
	virtual bool SetUniform2f(const std::string& name, eShaderType type, GLfloat value0, GLfloat value1);
	virtual bool SetUniform3f(const std::string& name, eShaderType type, GLfloat value0, GLfloat value1, GLfloat value2);
	virtual bool SetUniform4f(const std::string& name, eShaderType type, GLfloat value0, GLfloat value1, GLfloat value2, GLfloat value3);
	virtual bool SetUniform1fArray(const std::string& name, eShaderType type, int count, const float* data);
	virtual bool SetUniform1iArray(const std::string& name, eShaderType type, int count, const int* data);
	virtual bool SetUniform2fArray(const std::string& name, eShaderType type, int count, const float* data);
	virtual bool SetUniform3fArray(const std::string& name, eShaderType type, int count, const float* data);
	virtual bool SetGLMatrix(const std::string& name, eShaderType type,  GLenum matrix, GLenum transform);


	//only one context is required for all shaders
	static CGcontext m_context;
	static CGprofile m_vertexProfile;
	static CGprofile m_geometryProfile;
	static CGprofile m_fragmentProfile;

	CGprogram m_vertexHandle;
	CGprogram m_geometryHandle;
	CGprogram m_fragmentHandle;

	CgParamMap m_vertexParamMap;
	CgParamMap m_fragmentParamMap;
};

#endif
